/*
 * common.c - Functions shared by example applications
 *
 * Written 2002 by Werner Almesberger
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <tcpcp.h>

#include "util.h"


/* ----- Dump ICI ---------------------------------------------------------- */


void dump_ici(int s,const char *ici_file,int stdout_fd)
{
    void *ici;
    int fd;

    fprintf(stderr,"Attempting dump ...\n");
    fflush(stderr);
    ici = tcpcp_get(s);
    if (!ici) {
	perror("tcpcp_get");
	exit(1);
    }
    fd = strcmp(ici_file,"-") ? creat(ici_file,0666) : stdout_fd;
    if (fd < 0) {
	perror(ici_file);
	exit(1);
    }
    write_file(fd,ici,tcpcp_size(ici));
    if (fd != stdout_fd && close(fd) < 0) {
	perror(ici_file);
	exit(1);
    }
}


/* ----- Accept new connection --------------------------------------------- */


int accept_connection(int port)
{
    struct sockaddr_in addr;
    int s,s2;

    s = socket(PF_INET,SOCK_STREAM,0);
    if (s < 0) {
	perror("socket");
	exit(1);
    }
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port = htons(port);
    if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
	perror("bind");
	exit(1);
    }
    if (listen(s,1) < 0) {
	perror("listen");
	exit(1);
    }
    fprintf(stderr,"Waiting for connection ...\n");
    fflush(stdout);
    s2 = accept(s,NULL,NULL);
    if (s2 < 0) {
	perror("accept");
	exit(1);
    }
    (void) close(s);
    return s2;
}


/* ----- Resurrect connection from ICI ------------------------------------- */


int resurrect_connection(const char *file)
{
    int fd,size,s;
    void *ici;

    fd = strcmp(file,"-") ? open(file,O_RDONLY) : 0;
    if (fd < 0) {
	perror(file);
	exit(1);
    }
    ici = read_file(fd,&size);
    if (fd) (void) close(fd);
    fprintf(stderr,"Attempting to resurrect connection ...\n");
    fflush(stdout);
    s = tcpcp_create(ici);
    if (s < 0) {
	perror("tcpcp_create");
	exit(1);
    }
    return s;
}
