/*
 * cpshd.c - Simple connection-passing shell with "TELNET" server
 *
 * Written 2002 by Werner Almesberger
 */


#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include <tcpcp.h>

#include "util.h"


static char *ici_file = NULL;
static int stdout_fd,stderr_fd;


/* ----- Dump and exit ----------------------------------------------------- */



static int do_dump(char **argv)
{
    fprintf(stderr,"Attempting dump ...\n");
    fflush(stderr);
    (void) dup2(stderr_fd,2);
    dump_ici(0,ici_file,stdout_fd);
    fprintf(stderr,"Done. Waiting for signal.\n");
    fflush(stderr);
    while (1) pause();
}


/* ----- Command-line handling --------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-l telnet_port] ici_file\n\n",name);
    fprintf(stderr,"  -l  listen for incoming TCP connection on "
      "\"telnet_port\"\n");
    exit(1);
}


int main(int argc,char **argv)
{
    unsigned long telnet_port = 0;
    char *end;
    char c;
    int s;

    while ((c = getopt(argc,argv,"l:")) != EOF)
	switch (c) {
	    case 'l':
		telnet_port = strtoul(optarg,&end,10);
		if (*end || !telnet_port || (telnet_port & ~0xffff))
		    usage(argv[0]);
		break;
	    default:
		usage(argv[0]);
	}
    if (optind+1 != argc) usage(argv[0]);
    ici_file = argv[optind];
    shmod_add_command("dump",do_dump);
    if (telnet_port) s = accept_connection(telnet_port);
    else s = resurrect_connection(ici_file);
    fprintf(stderr,"Entering shell\n");
    fflush(stdout);
    stdout_fd = dup(1);
    if (stdout_fd < 0) {
	perror("dup");
	return 1;
    }
    stderr_fd = dup(2);
    if (stderr_fd < 0) {
	perror("dup");
	return 1;
    }
    if (dup2(s,0) < 0 || dup2(s,1) < 0 || dup2(s,2) < 0) {
	perror("dup2");
	return 1;
    }
    shmod_forced_tty = 0;
    shmod_catch_signals = 0;
    sh_mod(1,NULL);   
    return 0;
}
