/*
 * fileio.c - Load/write file, possibly from/to pipe
 *
 * Written 2002 by Werner Almesberger
 */


#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include "util.h"


#define BUF_INIT	8192
#define READ_MIN	4096


void *read_file(int fd,int *size)
{
    int buf_size = BUF_INIT;
    char *buf;

    buf = malloc(buf_size);
    if (!buf) {
	perror("malloc");
	exit(1);
    }
    *size = 0;
    while (1) {
	ssize_t got;

	got = read(fd,buf+*size,buf_size-*size);
	if (!got) return buf;
	if (got < 0) {
	    perror("read");
	    exit(1);
	}
	*size += got;
	if (buf_size-*size < READ_MIN) {
	    buf_size *= 2;
	    buf = realloc(buf,buf_size);
	    if (!buf) {
		perror("realloc");
		exit(1);
	    }
	}
    }
}


void write_file(int fd,void *buf,int size)
{
    while (size) {
	ssize_t wrote;

	wrote = write(fd,buf,size);
	if (wrote < 0) {
	    perror("write");
	    exit(1);
	}
	size -= wrote;
	buf = (char *) buf+wrote;
    }
}
